/*************************************************************************
  (c) Copyright.  Digital Equipment Corporation, 1995.  All Rights
  Reserved.

  Permission is hereby granted to use, copy, modify, or enhance this 
  software freely, as long as the foregoing copyright of Digital Equipment
  Corporation and this notice are retained on the software.  This 
  software may not be distributed or sublicensed for a fee.  Digital      
  makes this software available "AS IS" and without warranties of any
  kind.  
 *************************************************************************/

/*
* Module Name:
*   cm.h
*   
* Overview:
*   This describes the management interface to the Connection Manager (CM).
*
* Authors:
*   TLR - Theodore L. Ross
*
* Modification History:
*   Date       Name  Description 
*   17-Oct-94  TLR   Created.
*   19-Jan-94  TLR   Added LAN Emulation map list hook.
*
* Description:
*   This is the interface through which the CM is managed.  It enables
*   instances of the the CM to be initialized at system start-up time (or any
*   other time).
*
*   The CM provides connection management services on a per-adapter basis.
*   Thus if there is more than one physical adapter in a system,
*   there are multiple instantiations of the CM.  The management
*   interface provides calls to create instantiations of the CM module.
*
* Future Extensions:
*   This interface shall be extended with calls that allow management entities
*   to access the internal state of a CM instance.
*
* Call Sequence Requirements:
*
*   Creation of instances of the CM is a two-step process.  First, each
*   instance must be created using the cm_create call.  The instance must
*   then be configured using the cm_config call.  This two-step process
*   is needed because it breaks a circular dependency in the architecture (i.e.
*   other module instances need the CM handle during creation and the handles
*   of those module instances are needed during creation the CM instance).
*
*   The call sequence for each instance will look like the following:
*
*       <os_utilities created>..............OS Utilities are created.
*
*       cm_create...........................Create a CM module instance.  OS
*                                           handle is passed so create routine
*                                           can allocate memory.
*
*       <other module instances created>....Create other module instances using
*                                           the CM handle from the
*                                           cm_create.
*
*       cm_drv_config.......................Called during lower driver creation.
*
*       cm_svc_config.......................Called during SVC module creation.
*
*       cm_map_get..........................Called by the LAN Emulation Client
*                                           (if it exists) to gain access to
*                                           the CM's VC and ATM address lists.
*/

typedef void (*LINK_CALLBACK) (HANDLE context,
                               HANDLE link_handle);

/*++
* =============
* = cm_create =
* =============
*
* Overview:
*   Called once for each instance of the CM that is needed.  Called during
*   system initialization.
*
* Arguments:
*   os_handle       - (IN)  The OS handle to be used by this instance.
*   p_text          - (IN)  A constant text string to describe this instance.
*   p_cm_handle     - (OUT) The handle that describes the new instance.
*              
* Returns:
*   STATUS_K_SUCCESS    - Success, see postconditions.
*   STATUS_K_RESOURCES  - Insufficient resources to allocate new handle.
*
* Preconditions:
*   None
*
* Postconditions:
*   The cm_handle is valid.
--*/
STATUS cm_create(HANDLE  os_handle,
                 char   *p_text,
                 HANDLE *p_cm_handle);

/*++
* ==============
* = cm_map_get =
* ==============
*
* Overview:
*   Gets pointers to this CM instance's mapping lists (VC list and ATM address
*   list).  This function is called by the LAN Emulation Client module during
*   module creation.
*
* Arguments:
*   cm_handle             - (IN)  Handle returned by cm_create.
*   link_update_callback  - (IN)  Callback for MAC address link updates.
*   link_remove_callback  - (IN)  Callback for MAC address link removal.
*   callback_context      - (IN)  Context to be passed back with the callbacks.
*   p_vc_list             - (OUT) VC List handle for this cm instance.
*   p_atm_list            - (OUT) ATM Address List handle for this cm instance.
*              
* Returns:
*   None
*
* Preconditions:
*   The cm_handle must be valid.
*
* Postconditions:
*   None
--*/
void cm_map_get(HANDLE         cm_handle,
                LINK_CALLBACK  link_update_callback,
                LINK_CALLBACK  link_remove_callback,
                HANDLE         callback_context,
                HANDLE        *p_vc_list,
                HANDLE        *p_atm_list);

/*++
* ==============
* = cm_destroy =
* ==============
*
* Overview:
*   Destroys a previously created CM instance.
*
* Arguments:
*   cm_handle       - (IN) The handle returned by cm_create.
*              
* Returns:
*   None
*
* Preconditions:
*   None
*
* Postconditions:
*   The cm_handle is invalid.
--*/
void cm_destroy (HANDLE  cm_handle);

/*++
* =================
* = cm_link_event =
* =================
*
* Overview:
*   Notifies the CM of events that may affect the link state.  This call is
*   invoked from the lower device driver to notify the CM of changes to the
*   physical link state and of port resets.
*
* Arguments:
*   cm_handle   - (IN) CM handle associated with this driver instance.
*   link_event  - (IN) Link event
*              
* Returns:
*   None.
*
* Preconditions:
*   None.
*
* Postconditions:
*   None.
--*/
void cm_link_event (HANDLE           cm_handle,
                    ATM_LINK_EVENT   link_event);


void cm_event_mask_set (HANDLE   cm_handle,
                        UINT32   event_mask);

void conn_vcid_get (HANDLE  conn_handle,
                    UINT8  *p_vpi,
                    UINT16 *p_vci);

