/*************************************************************************
  (c) Copyright.  Digital Equipment Corporation, 1995.  All Rights
  Reserved.

  Permission is hereby granted to use, copy, modify, or enhance this 
  software freely, as long as the foregoing copyright of Digital Equipment
  Corporation and this notice are retained on the software.  This 
  software may not be distributed or sublicensed for a fee.  Digital      
  makes this software available "AS IS" and without warranties of any
  kind.  
 *************************************************************************/

/*
* Module Name:
*   cm_svc.h
*
* Overview:
*   This module describes the interface between the connection manager and
*   svc module.
*
* Authors:
*   JAD - John A. DeNisco
*   TLR - Theodore L. Ross
*
* Modification History:
*   Date       Name  Description
*   14-Oct-94  JAD   Created.
*   02-Nov-94  JAD   Incorporated internal review inputs.
*   30-Jan-95  JAD   Changed cm_sig to cm_svc.
*   25-Sep-95  TLR   Merged with svc_cm.h to make one interface file.
*
* Description:
*   This module describes functions exported by the connection manager for
*   use by the svc module. These functions take the form of indications and
*   confirmations.  It additionally defines functions exported by the
*   svc module for use by the connection manager.  These functions take the
*   form of requests and responses.
*
*/

/****************************************************************************
 * Routines implemented in CM module.  Indications and Confirmations
 ***************************************************************************/

/*++
* =================
* = cm_svc_config =
* =================
*
* Overview:
*   Provides the CM instance with the signaling handle.
*
* Arguments:
*   cm_handle   - (IN)  Handle returned by cm_create.
*   svc_handle  - (IN)  Handle of the Signaling module instance
*              
* Returns:
*   None
*
* Preconditions:
*   The cm_handle must be valid.
*
* Postconditions:
*   None
--*/
void cm_svc_config (HANDLE  cm_handle,
                    HANDLE  svc_handle);

/*++
* ===================
* = cm_svc_conn_ind =
* ===================
*
* Overview:
*  This function is the virtual connection" indication. It is initiated
*  by the svc module upon the receipt of a "setup" message.
*
*  When this function is called, the connection manager must try to setup the
*  virtual connection. If the connection manager CAN set up the connection,
*  then it must call the "svc_cm_conn_rsp" function. If the connection manager
*  CANNOT setup the connection, then it must call "svc_cm_rel_req".
*
* Arguments:
*  cm_handle        - (IN) The handle of the signaling instance to be used
*                          for this connect request.
*
*  call_provider_id - (IN) ID used to identify the provider of this call
*                          instance.
*
*
*  p_conn_info      - (IN) Pointer to the connection information.
*
* Returns:
*  None
*
* Preconditions:
*  None
*
* Postconditions:
*  VC will go into the pending state.
*
--*/

void cm_svc_conn_ind (HANDLE     cm_handle,
                      CALL_ID    call_provider_id,
                      CONN_INFO *p_conn_info);

/*++
* =======================
* = cm_svc_conn_sts_ind =
* =======================
*
* Overview:
*  This function is a connection status indication. It is initiated by the svc
*  module upon the receipt of a "call proceeding" message. The call proceeding
*  message contains the Virtual Connection Indicator. The connection manager
*  uses this to complete connection setup.  
*
* Arguments:
*  cm_handle        - (IN) The handle of the signaling instance to be used
*                          for this connect request.
*
*  call_user_id     - (IN) ID used to identify the user of this call instance.
*
*  call_provider_id - (IN) ID used to identify the provider of this call
*                          instance.
*
*  p_conn_status    - (IN) Pointer to the connection status information.
*
*  event            - (IN) Event type that caused the status indication.
*
* Returns:
*  None
*
* Preconditions:
*  None.
*
* Postconditions:
*  None.
*
--*/

void cm_svc_conn_sts_ind (HANDLE       cm_handle,
                          CALL_ID      call_user_id,
                          CALL_ID      call_provider_id,
                          CONN_STATUS *p_conn_status,
                          EVENT_TYPE   event);

/*++
* ===================
* = cm_svc_conn_cfm =
* ===================
*
* Overview:
*  This function is the virtual connection" confirmation. It is initiated by
*  the svc module when it receives a "connect acknowledge" message. To
*  process the virtual connection confirmation, the connection manager must
*  complete the virtual connection setup.
*
* Arguments:
*
*  cm_handle        - (IN) The handle of the signaling instance to be used
*
*  call_user_id     - (IN) ID used to identify the user of this call instance.
*
*  call_provider_id - (IN) ID used to identify the provider of this call
*                          instance.
*
*  p_conn_status    - (IN) Pointer to the connection status information.
*
* Returns:
*  None
*
* Preconditions:
*  None
*
* Postconditions:
*  VC setup is now complete.
*
--*/

void cm_svc_conn_cfm (HANDLE       cm_handle,
                      CALL_ID      call_user_id,
                      CALL_ID      call_provider_id,
                      CONN_STATUS *p_conn_status);

/*++
* ==================
* = cm_svc_rel_ind =
* ==================
*
* Overview:
*  This function is the release" indication. This indication is initiated by
*  the svc module upon the receipt of a "release" message.
*
*  The connection manager processes the release indication releasing the
*  virtual connection.
*
* Arguments:
*
*  cm_handle        - (IN) The handle of the signaling instance to be used
*
*  call_user_id     - (IN) ID used to identify the user of this call instance.
*
*  call_provider_id - (IN) ID used to identify the provider of this call
*                          instance.
*
*  p_rel_info       - (IN) The release information.
*
* Returns:
*  None
*
* Preconditions:
*  None
*
* Postconditions:
*  Virtual connection is released.
*
--*/

void cm_svc_rel_ind (HANDLE        cm_handle,
                     CALL_ID       call_user_id,
                     CALL_ID       call_provider_id,
                     RELEASE_INFO *p_rel_info);

/*++
* ======================
* = cm_svc_restart_ind =
* ======================
*
* Overview:
*  This function is the restart" indication. It is initiated by the svc module
*  upon the receipt of a "restart" message.
*
*  The connection manager will restart a virtual connection, or all associated
*  with the virtual connection, upon this indication. Once the connection/
*  connections are released the connection manager will call
*  "svc_cm_restart_rsp" function.
*
* Arguments:
*
*  cm_handle  - (IN) The handle of the signaling instance to be used
*
*  reason     - (IN) The restart reason.
*
* Returns:
*  None
*
* Preconditions:
*  None.
*
* Postconditions:
*  Virtual connection/connections are released.
*
--*/

void cm_svc_restart_ind (HANDLE      cm_handle,
                         RST_REASON  reason);

/*++
* ========================
* = cm_svc_add_party_ind =
* ========================
*
* Overview:
*  This function is the add party indication. It initiated by the svc module
*  upon the receipt of an "add party" message. 
*
*  The connection manager will try to add a party to a virtual connection,
*  upon this indication. If the party CAN be added to the virtual connection
*  the connection manager will call the "svc_cm_add_party_rsp" function. If
*  the party CAN NOT be added to the connection the connection will call the
*  "svc_cm_add_party_rej_req" function.
*
* Arguments:
*  cm_handle        - (IN) The handle to the instance to be used for this
*                          add party request.
*
*  call_user_id     - (IN) ID used to identify the user of this call instance.
*
*  call_provider_id - (IN) ID used to identify the provider of this call
*                          instance.
*
*  p_add_pty_info   - (IN) Pointer to the add party information.
*
* Returns:
*  None
*
* Preconditions:
*  None.
*
* Postconditions:
*  The party has been added to the connection.
*
--*/

void cm_svc_add_party_ind (HANDLE        cm_handle,
                           CALL_ID       call_user_id,
                           CALL_ID       call_provider_id,
                           ADD_PTY_INFO *p_add_pty_info);

/*++
* ========================
* = cm_svc_add_party_cfm =
* ========================
*
* Overview:
*  This function is the add party confirmation. It is initiated by the svc
*  module upon receipt of an "add party acknowledge" message.
*
*  The connection manager will complete the the addition of a party to a
*  virtual connection upon this confirmation. This confirmation is initiated
*  by receipt of an add party acknowledge message.
*
* Arguments:
*  cm_handle        - (IN) The handle to the instance to be used for this
*                          add party request.
*
*  call_user_id     - (IN) ID used to identify the user of this call instance.
*
*  call_provider_id - (IN) ID used to identify the provider of this call
*                          instance.
*
*  p_add_pty_status - (IN) Pointer to the add party information.
*
* Returns:
*  None
*
* Preconditions:
*  None.
*
* Postconditions:
*  Add party request is completed.
*
--*/

void cm_svc_add_party_cfm (HANDLE          cm_handle,
                           CALL_ID         call_user_id,
                           CALL_ID         call_provider_id,
                           ADD_PTY_STATUS *p_add_pty_status);

/*++
* ============================
* = cm_svc_add_party_rej_ind =
* ============================
*
* Overview:
*  This function is the add party reject indication. It is initiated by the
*  signaling module upon receipt of an add party reject message.
*
*  The connection manager will eliminate the add party request upon this
*  indication.
*
* Arguments:
*  cm_handle        - (IN) The handle to the instance to be used for this
*                           add party request.
*
*  call_user_id      - (IN) ID used to identify the user of this call instance.
*
*  call_provider_id  - (IN) ID used to identify the provider of this call
*                           instance.
*
*  p_drop_pty_info   - (IN) Pointer to the add party information.
*
* Returns:
*  None.
*
* Preconditions:
*  None.
*
* Postconditions:
*  Add party request is eliminated.
*
--*/

void cm_svc_add_party_rej_ind (HANDLE         cm_handle,
                               CALL_ID        call_user_id,
                               CALL_ID        call_provider_id,
                               DROP_PTY_INFO *p_drop_pty_info);

/*++
* =========================
* = cm_svc_drop_party_ind =
* =========================
*
* Overview:
*  This is the drop party" indication. It is initiated by the svc module when
*  upon receipt of a drop party message. 
*
*  The connection manager will drop a party from a virtual connection, upon
*  this indication. Once the drop party is completed the connection manager
*  will call the sig_drop_party_rsp function.
*
* Arguments:
*  cm_handle        - (IN) The handle to the instance to be used for this
*                          drop party request.
*
*  call_user_id     - (IN) ID used to identify the user of this call instance.
*
*  call_provider_id - (IN) ID used to identify the provider of this call
*                          instance.
*
*  p_drop_pty_info  - (IN) Pointer to the drop party information.
*
* Returns:
*  None
*
* Preconditions:
*  None.
*
* Postconditions:
*  A party is dropped from the connection.
*
--*/

void cm_svc_drop_party_ind (HANDLE         cm_handle,
                            CALL_ID        call_user_id,
                            CALL_ID        call_provider_id,
                            DROP_PTY_INFO *p_add_pty_info);





/****************************************************************************
 * Routines implemented in SVC module.  Requests and Responses
 ***************************************************************************/

/*++
* ===================
* = svc_cm_conn_req =
* ===================
*
* Overview:
*  This function is a virtual connection request. It is initiated by the
*  connection manager when it needs to setup a virtual connection.
*
* Arguments:
*  svc_handle   - (IN) The handle of the signaling instance to be used
*                      for this connect request.
*
*  call_user_id - (IN) ID used to identify the user of this call instance.
*
*  p_conn_info  - (IN) Pointer to the connection information.
*
* Returns:
*  STATUS_K_SUCCESS - Success, see postconditions.
*  STATUS_K_FAILURE - Not OK returned from Q93.B module.
*
* Preconditions:
*  None.
*
* Postconditions:
*  Setup message is transmitted.
--*/

STATUS svc_cm_conn_req (HANDLE     svc_handle,
                        CALL_ID    call_user_id,
                        CONN_INFO *p_conn_info);

/*++
* ===================
* = svc_cm_conn_rsp =
* ===================
*
* Overview:
*  This function is the virtual connection response. It is initiated by the
*  connection manager when it is able to succesfully setup a virtual
*  connection.
*
* Arguments:
*  svc_handle       - (IN) The handle of the signaling instance to be used
*                          for this connect request.
*
*  call_user_id     - (IN) ID used to identify the user of this call instance.
*
*  call_provider_id - (IN) ID used to identify the provider of this call
*                          instance.
*
*  p_conn_status    - (IN) Pointer to the connection status information.
*
* Returns:
*  STATUS_K_SUCCESS - Success, see postconditions.
*  STATUS_K_FAILURE - Not OK returned from Q93.B module.
*
* Preconditions:
*  None.
*
* Postconditions:
*  Connect acknowledge message is transmitted.
--*/

STATUS svc_cm_conn_rsp (HANDLE       svc_handle,
                        CALL_ID      call_user_id,
                        CALL_ID      call_provider_id,
                        CONN_STATUS *p_conn_status);

/*++
* ==================
* = svc_cm_rel_req =
* ==================
*
* Overview:
*  This function is a release request. It is initiated by the connection
*  manager when it needs to release a virtual connection.
*
* Arguments:
*  svc_handle       - (IN) The handle of the instance to be used for this
*                          release request.
*
*  call_user_id     - (IN) ID used to identify the user of this call instance.
*
*  call_provider_id - (IN) ID used to identify the provider of this call
*                          instance.
*
*  p_rel_info       - (IN) Information associated with this release.
*
* Returns:
*  STATUS_K_SUCCESS - Success, see postconditions.
*  STATUS_K_FAILURE - Not OK returned from Q93.B module.
*
* Preconditions:
*  None.
*
* Postconditions:
*  Release message is transmitted.
--*/

STATUS svc_cm_rel_req (HANDLE        svc_handle,
                       CALL_ID       call_user_id,
                       CALL_ID       call_provider_id,
                       RELEASE_INFO *p_rel_info);

/*++
* ========================
* = svc_cm_add_party_req =
* ========================
*
* Overview:
*  This function is the add party request. It initiated by the connection
*  manager when it needs to add a party to a virtual connection.
*
* Arguments:
*  svc_handle       - (IN) The handle to the instance to be used for this
*                          add party request.
*
*  call_user_id     - (IN) ID used to identify the user of this call instance.
*
*  call_provider_id - (IN) ID used to identify the provider of this call
*                          instance.
*
*  p_add_pty_info   - (IN) Pointer to the add party information.
*
* Returns:
*  STATUS_K_SUCCESS - Success, see postconditions.
*  STATUS_K_FAILURE - Not OK returned from Q93.B module.
*
* Preconditions:
*  None.
*
* Postconditions:
*  The add party message is transmitted.
--*/

STATUS svc_cm_add_party_req (HANDLE        svc_handle,
                             CALL_ID       call_user_id,
                             CALL_ID       call_provider_id,
                             ADD_PTY_INFO *p_add_pty_info);

/*++
* ========================
* = svc_cm_add_party_rsp =
* ========================
*
* Overview:
*  This function is an add party response. This function is initiated by the
*  connection manager if it was able to succesfully add a party to a virtual
*  connection. 
*
* Arguments:
*  svc_handle       - (IN) The handle to the instance to be used for this
*                          add party request.
*
*  call_user_id     - (IN) ID used to identify the user of this call instance.
*
*  call_provider_id - (IN) ID used to identify the provider of this call
*                          instance.
*
*  p_add_pty_status - (IN) Pointer to the add party status information.
*
* Returns:
*  STATUS_K_SUCCESS - Success, see postconditions.
*  STATUS_K_FAILURE - Not OK returned from Q93.B module.
*
* Preconditions:
*  None.
*
* Postconditions:
*  Add party acknowledge is transmitted.
--*/

STATUS svc_cm_add_party_rsp (HANDLE          svc_handle,
                             CALL_ID         call_user_id,
                             CALL_ID         call_provider_id,
                             ADD_PTY_STATUS *p_add_pty_status);

/*++
* ============================
* = svc_cm_add_party_rej_req =
* ============================
*
* Overview:
*  This function is the an add party reject request. It is initiated by the
*  connection manager calls function when a party can not be added to virtual
*  connection.
*
* Arguments:
*  svc_handle       - (IN) The handle to the instance to be used for this
*                          add party request.
*
*  call_user_id     - (IN) ID used to identify the user of this call instance.
*
*  call_provider_id - (IN) ID used to identify the provider of this call
*                          instance.
*
*  p_drop_pty_info  - (IN) Pointer to the drop party information.
*
* Returns:
*  STATUS_K_SUCCESS - Success, see postconditions.
*  STATUS_K_FAILURE - Not OK returned from Q93.B module.
*
* Preconditions:
*  None.
*
* Postconditions:
*  Add party reject message is transmitted.
--*/

STATUS svc_cm_add_party_rej_req (HANDLE         svc_handle,
                                 CALL_ID        call_user_id,
                                 CALL_ID        call_provider_id,
                                 DROP_PTY_INFO *p_drop_pty_info);

/*++
* =========================
* = svc_cm_drop_party_req =
* =========================
*
* Overview:
*  This function is the drop party request. It is initiated by the connection
*  manager when it needs to drop a party from a virtual connection.
*
* Arguments:
*  svc_handle       - (IN) The handle to the instance to be used for this
*                          drop party request.
*
*  call_user_id     - (IN) ID used to identify the user of this call instance.
*
*  call_provider_id - (IN) ID used to identify the provider of this call
*                          instance.
*
*  p_drop_pty_info  - (IN) Pointer to the drop party information.
*
* Returns:
*  STATUS_K_SUCCESS - Success, see postconditions.
*  STATUS_K_FAILURE - Not OK returned from Q.93B.
*
* Preconditions:
*  None.
*
* Postconditions:
*  A drop party message is transmitted.
--*/

STATUS svc_cm_drop_party_req (HANDLE         svc_handle,
                              CALL_ID        call_user_id,
                              CALL_ID        call_provider_id,
                              DROP_PTY_INFO *p_drop_pty_info);

/*++
* ======================
* = svc_cm_restart_req =
* ======================
*
* Overview:
*  This function is the restart request. It is initiated by the connection
*  manager when it needs to restart virtual connections.
*
* Arguments:
*  svc_handle - (IN) The handle of the instance to be used for this
*                    release request.
*
*  reason     - (IN) The restart reason.
*
* Returns:
*  STATUS_K_SUCCESS - Success, see postconditions.
*  STATUS_K_FAILURE - Not OK returned from Q93.B module.
*
* Preconditions:
*  None.
*
* Postconditions:
*  Restart message is transmitted.
--*/

STATUS svc_cm_restart_req (HANDLE svc_handle);
