/*************************************************************************
  (c) Copyright.  Digital Equipment Corporation, 1995.  All Rights
  Reserved.

  Permission is hereby granted to use, copy, modify, or enhance this 
  software freely, as long as the foregoing copyright of Digital Equipment
  Corporation and this notice are retained on the software.  This 
  software may not be distributed or sublicensed for a fee.  Digital      
  makes this software available "AS IS" and without warranties of any
  kind.  
 *************************************************************************/

/*
* Module Name:
*   utl_os.c
*   
* Abstract:
*   This file implements the general operating system specific utilities for
*   the Windows NT operating system. This file cannot be used by a device
*   driver, but is useful for application level testing and as a template.
*                                                                             
* Authors:
*   DMW - Douglas M. Washabaugh
*
* Modification History:
*   Date       Name  Description 
*   08-Aug-94  DMW   Created.
*   07-Dec-94  DMW   Updated with tons of changes and additions.
*   10-Jan-95  DMW   os_timer_cancel and os_timer_dealloc now handle null
*                      handles.  Removed physical memory routines.
*   10-Jan-95  DMW   Remove obsolete phys mem routines, added mem statistics.
*   16-Feb-95  PJR   Updated os_create with an additional parameter, p_text.
*   16-Mar-95  DMW   Added two new routines os_buff_hdr_copy, os_buff_hdr_get.
*   14-Jun-95  DMW   Removed volatile keywords.
*
* Description:
*   There are 4 catagories of routines:
*     - Timers
*     - Memory Allocation/Deallocation
*     - Register Access
*     - Logging
*
* Linking:
*   Link with winmm.lib.
*/

#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
#include <assert.h>
#include "g_types.h"
#include "codes.h"
#include "utl.h"
#include "utl_os.h"
#include "port_sim.h"

/*++
* Define a structure which resides in allocated physical memory.
*   This structure is used to simplify the returning of memory, so that the 
*   caller does not need to know the actual address of allocated memory  
*   (as opposed to the aligned address).
--*/

typedef struct
   {
   UINT32 not_used;  /* Kept as a hint for other implementations. */
   } MEM_HDR;

/*++
* Define the structure to hold the information for an allocated timer.
*
*   p_next -
*     Link to the next timer in the list.
*
*   os_handle -
*     The handle for the instantiation with which this timer is associated.
*     This is provided as a convenience so that the caller only has to deal 
*     with timers instead of timers and instantiations.
*
*   callback -
*     Pointer to the callback routine to call when the timer expires.
*
*   context -
*     Value to pass to the callback routine.
*
*   time_in_msec -
*     Time remaining on this timer if active.
*
*   active -
*     Boolean flag indicating whether this timer is running.
*
--*/

typedef struct _timer
   {
   struct _timer     *p_next;
   HANDLE             os_handle;
   OS_TIMER_CALLBACK  callback;
   HANDLE             context;
   UINT32             time_in_msec;
   BOOLEAN            active;
   } TIMER;

/* Define a type for a list of timers. */

SIMPLE_LIST  (TIMER, TIMER_LIST);

/*++
* Define structure to hold information for each instantiation.
*
*   adap_handle -
*     A value that was passed by the creater of the module.  This is not
*     currently used by this implementation, but is kept here as a placeholder.
*
*   sim_handle -
*     Handle returned from sim_create.
*
*   p_sys_log -
*     Pointer to the FILE structure for general logging.  This is defined
*     at instantiation time, and must be deallocated at destruction time.
*
*   timer_list -
*     A list of allocated timers.  These timers are not necessarily active.
*
*   timer_resolution -
*     The resolution of the timer in milliseconds.  This value is negotiated
*     with the OS and must be canceled when the instantiation is destroyed.
*
*   pool_size -
*     The maximum number of bytes that can be allocated concurrently.
*
*   peak_bytes -
*     The peak number of bytes allocated since the instantiation was created.
*
*   allocated_bytes -
*     The current number of outstanding allocated bytes.
--*/

typedef struct
   {
   HANDLE       adap_handle;
   HANDLE       sim_handle;
   FILE        *p_sys_log;
   TIMER_LIST   timer_list;
   UINT         timer_resolution;
   UINT32       pool_size;
   UINT32       peak_bytes;
   UINT32       allocated_bytes;
   } OS_INFO;


/*++
* =============
* = os_create =
* =============
*
* Description:
*   Two logging files are opened, sys_log.txt and err_log.txt.  Any existing 
*   contents of the files are lost.
*
* Testing:
*   o Test with files sys_log.txt and err_log.txt in the working directory.
*     Make sure that the create completes successfully.
*
*   o Most testing must be in conjunction with other routines.
--*/

STATUS os_create (HANDLE  adap_handle,
                  char   *p_text,
                  HANDLE *p_os_handle)
   {
   OS_INFO *p_os_info;
   TIMECAPS time_caps;  /* Contains Windows NT timer information. */
   UINT     resolution;
   STATUS   status;

   /* Allocate and clear memory for instantiation, return if failure. */

   p_os_info = malloc (sizeof (*p_os_info));
   if (p_os_info == NULL) return (STATUS_K_RESOURCES);

   (void) memset (p_os_info, '\0', sizeof (*p_os_info));

   *p_os_handle = (HANDLE) p_os_info;
      
   /* Save the adapter handle. */

   p_os_info->adap_handle = adap_handle;
   
   /* Initialize the list of timers. */

   utl_list_init (p_os_info->timer_list);

   /* Open the file and write the appropriate text. */
   /*   Return if failure.                          */

   p_os_info->p_sys_log = fopen ("sys_log.txt", "w");

   if (p_os_info->p_sys_log == NULL)
      {
      os_destroy (*p_os_handle);
      return (STATUS_K_RESOURCES);
      }

   /* Initialize the memory configuration.                                */
   /*   Init the pool size to a very large value, so all mem allocs work. */

   p_os_info->pool_size       = 0xFFFFFFFF;
   p_os_info->peak_bytes      = 0;
   p_os_info->allocated_bytes = 0;

   /* Create a simulation instantiation. */

   status = sim_create (p_os_info, &p_os_info->sim_handle);
   if (status != STATUS_K_SUCCESS) 
      {
      os_destroy (*p_os_handle);
      return (STATUS_K_RESOURCES);
      }

   /* Return the handle. */

   return (STATUS_K_SUCCESS);
   }

/*++
* ==============
* = os_destroy =
* ==============
*
* Description:
*   All timers are cancelled and deallocated.  The minimum resolution on the
*   timer is unreserved.  The log files are closed.
*
* Testing:
*   o call with no timers allocated.
*   o call with some timers allocated, but all "inactive".
*   o call with at least one timer "active".
--*/

void os_destroy (HANDLE os_handle)
   {
   OS_INFO *p_os_info;
   TIMER   *p_timer;
   
   /* Get the pointer to the instantiation. */

   p_os_info = (OS_INFO *) os_handle;
   if (p_os_info == NULL) return;

   /* Deallocate the list of timers. */

   utl_list_traverse (p_os_info->timer_list, p_timer)
      {
      free (p_timer);
      }

   /* Cancel the minimum timer resolution. */

   timeEndPeriod (p_os_info->timer_resolution);

   /* Close the log files. */

   if (p_os_info->p_sys_log != 0) fclose (p_os_info->p_sys_log);

   /* Destroy the simulation instantiation. */

   sim_destroy (p_os_info->sim_handle);

   /* Deallocate the memory used for the instantiation. */

   free (p_os_info);
   }

/*++
* ============
* = os_delay =
* ============
*
* Description:
*   Not implemented in the test version.
*
* Testing:
--*/
void os_delay (HANDLE os_handle,
               UINT32 delay)
   {
   }

/*++
* ==================
* = os_timer_alloc =
* ==================
*
* Description:
*   Allocates and initializes a timer.  The timer is linked onto a list so
*   that it can be "killed" when the os_destroy is called.
*
* Testing:
*   o Make sure that 3 timers can be allocated and deallocated [in opposite
*     order].  Allocate timer A, B, C, and then deallocate C, B, A.
--*/
STATUS os_timer_alloc (HANDLE             os_handle,
                       OS_TIMER_CALLBACK  callback,
                       HANDLE             context,
                       HANDLE            *p_timer_handle)
   {
   OS_INFO *p_os_info;
   TIMER   *p_timer;

   /* Get a pointer to the instantiation. */

   p_os_info = (OS_INFO *) os_handle;

   /* Allocate a timer. */

   p_timer = malloc (sizeof (*p_timer));
   if (p_timer == NULL) return (STATUS_K_RESOURCES);
   
   /* Initialize the timer. */

   p_timer->os_handle    = os_handle;
   p_timer->callback     = callback;
   p_timer->context      = context;
   p_timer->active       = FALSE;
   p_timer->time_in_msec = 0;

   /* Link the timer into the timer list. */
   
   utl_list_add (p_os_info->timer_list, p_timer);
   
   /* Return the timer handle. */

   *p_timer_handle = (HANDLE) p_timer;

   /* Return success. */

   return (STATUS_K_SUCCESS);
   }

/*++
* ====================
* = os_timer_dealloc =
* ====================
*
* Description:
*   The timer is first cancelled, and then it is deallocated.
*
* Testing:
*   Tested while testing os_timer_alloc.
--*/

void os_timer_dealloc (HANDLE timer_handle)
   {
   OS_INFO *p_os_info;
   TIMER   *p_timer;

   /* Get a pointer to the instantiation. */

   p_timer = (TIMER *) timer_handle;
   if (p_timer == NULL) return;

   /* Cancel the timer. */

   (void) os_timer_cancel (timer_handle);
   
   /* Remove the timer from the timer list and free its memory. */

   p_os_info = (OS_INFO *) p_timer->os_handle;
   utl_list_delete (p_os_info->timer_list, p_timer);

   free (p_timer);
   }

/*++         
* ================
* = os_timer_set =
* ================
*
* Description:
*   The timer is set to callback to a routine in this module.  That routine
*   dispatches to the appropriate callback.
*
* Testing:
*   o Test two concurrent timers, one 1 second and one 5 seconds.  Have each
*     reschedule themselves for a total test time of 10 seconds.  Ensure that
*     the 1 second is called more than the 5 second one.
--*/
STATUS os_timer_set (HANDLE timer_handle,
                     UINT32  time)
   {
   TIMER   *p_timer;
   OS_INFO *p_os_info;

   /* Get pointer to the timer and the timer's instantiation. */

   p_timer   = (TIMER *)   timer_handle;
   p_os_info = (OS_INFO *) p_timer->os_handle;

   /* Cancel the outstanding timer, in case its still active. */

   (void) os_timer_cancel (timer_handle);

   p_timer->time_in_msec = time;
   p_timer->active       = TRUE;

   return STATUS_K_SUCCESS;
   }

/*++
* ===================
* = os_timer_cancel =
* ===================
*
* Description:
*   The timer is only canceled if it is active.  This is how the Microsoft
*   documentation recommends that a timer be canceled.
*
* Testing:
*   o Cancel a timer that has already expired
*   o Cancel a timer that has not yet expired, but leave a second timer
*     active.
--*/

STATUS os_timer_cancel (HANDLE timer_handle)
   {
   TIMER *p_timer;

   /* Return if the timer handle is invalid. */

   p_timer = (TIMER *) timer_handle;
   if (p_timer == NULL) return (STATUS_K_NO_WORK_TO_DO);

   /* Stop the timer if it is running, mark it as stopped. */

   p_timer->active       = FALSE;
   p_timer->time_in_msec = 0;
   }


#define RESOLUTION  100


/* Entry point for test routine.  This function is called to advance the time
 * in a software simulation.
 */

void os_advance_time (HANDLE os_handle,
                      UINT32 time)
   {
   TIMER   *p_timer;
   OS_INFO *p_os_info;
   UINT32   unit_time;

   p_os_info = (OS_INFO *) os_handle;

   for (; time > 0; time = (time > RESOLUTION) ? time - RESOLUTION : 0)
      {
      unit_time = MIN (time, RESOLUTION);
      utl_list_traverse (p_os_info->timer_list, p_timer)
         {
         if (p_timer->active)
            {
            if (p_timer->time_in_msec <= unit_time)
               {
               p_timer->active       = FALSE;
               p_timer->time_in_msec = 0;
               p_timer->callback(p_timer->context);
               }
            else
               {
               p_timer->time_in_msec -= unit_time;
               }
            }
         }
      }
   }

/*++
* ==============
* = os_reg_set =
* ==============
*
* Description:
*   This implementation assumes that the io_handle is actually the base of 
*   memory that has been previously allocated.
*
* Testing: 
*   Write a register and read it back with os_reg_get.
--*/

void os_reg_set (HANDLE  os_handle, 
                 UINT32 *p_reg, 
                 UINT32  value)
   {
   OS_INFO *p_os_info;

   /* Get a pointer to this instantiation. */

   p_os_info = (OS_INFO *) os_handle;

   /* Perform any necessary actions to roughly emulate the hardware. */
   
   sim_reg_set (p_os_info->sim_handle, (UINT32) p_reg, value);
   }
   
/*++
* ==============
* = os_reg_get =
* ==============
*
* Description:
*   This implementation assumes that the io_handle is actually the base of 
*   memory that has been previously allocated.
*
* Testing:
*   - Tested while testing os_reg_set.
--*/

UINT32 os_reg_get (HANDLE  os_handle,
                   UINT32 *p_reg)
   {
   OS_INFO        *p_os_info;
   volatile UINT32 tmp;

   /* Get a pointer to this instantiation. */

   p_os_info = (OS_INFO *) os_handle;

   sim_reg_get (p_os_info->sim_handle, (UINT32) p_reg, (UINT32 *) &tmp);
   return (tmp);
   }
   
/*++
* ================
* = os_mem_alloc =
* ================
*
* Description:
*   Just uses the basic C run-time call.
*
* Test Cases:
*   - Allocate an arbitrary size of memory and free it by calling
*     os_mem_dealloc.  Attempt to access 1st and last byte of allocated
*     memory.
*
*   - Print out the allocated memory to ensure that it is zero.
*
*   - Allocate more memory than is in the pool.
--*/

void *os_mem_alloc (HANDLE os_handle,
                    UINT32 size)
   {
   UINT32  *p_allocated_mem, i;
   UINT8   *p_mem;
   void    *p_user_mem;
   OS_INFO *p_os_info;

   /* Allocate the memory if there is room in the pool. */

   p_os_info = (OS_INFO *) os_handle;

   if ((p_os_info->allocated_bytes + size) > p_os_info->pool_size)
      return (NULL);

   p_allocated_mem  = (UINT32 *) malloc ((size_t) (size + sizeof (UINT32)));
   *p_allocated_mem = size;
   
   p_user_mem       = (p_allocated_mem + 1);

   /* Update memory statistics. */

   p_os_info->allocated_bytes += size;
   if (p_os_info->allocated_bytes > p_os_info->peak_bytes)
      p_os_info->peak_bytes = p_os_info->allocated_bytes;
         
   /* Clear each byte of the allocated memory.                                */
   /*   Other implementations may want to clear memory in a more optimal way. */

   p_mem = (UINT8 *) (p_user_mem);
   
   for (i = 0; i < size; i++)
     *p_mem++ = 0;
      
   return (p_user_mem);
   }

/*++
* ==================
* = os_mem_dealloc =
* ==================
*
* Description:
*   Just uses the basic C run-time call.
*
* Testing: 
*   Tested implicitely when testing os_mem_alloc
--*/

void os_mem_dealloc (HANDLE os_handle,
                     void   *p_user_mem)
   {
   OS_INFO *p_os_info;
   UINT32   size;
   void    *p_alloc_mem;

   /* Update memory statistics. */

   p_os_info = (OS_INFO *) os_handle;
   
   p_alloc_mem = (void *) ((UINT32 *) p_user_mem - 1);
   size        = *((UINT32 *) p_alloc_mem);

   p_os_info->allocated_bytes -= size;

   free (p_alloc_mem);
   }

/*++
* =================
* = os_mem_config =
* =================
*
* Description:
*   Just stores the value.
*
* Testing: 
*   - Allocate more memory than the pool size, should fail.
*   - Allocate less memory than the pool size should fail.
--*/

void os_mem_config (HANDLE  os_handle,
                    UINT32  pool_size)
   {
   OS_INFO *p_os_info;

   /* Store the pool size in the instantiation structure. */

   p_os_info            = (OS_INFO *) os_handle;
   p_os_info->pool_size = pool_size;
   }

/*++
* ================
* = os_mem_stats =
* ================
*
* Description:
*   Returns the memory statistics to the user buffer.
*
* Testing: 
*   Implicitely tested.
--*/

void os_mem_stats (HANDLE  os_handle,
                   UINT32  *p_peak_bytes,
                   UINT32  *p_allocated_bytes)
   {
   OS_INFO *p_os_info;

   p_os_info = (OS_INFO *) os_handle;

   *p_peak_bytes      = p_os_info->peak_bytes;
   *p_allocated_bytes = p_os_info->allocated_bytes;
   }
/*++
* ============
* = os_print =
* ============
*
* Description:
*   This routine appends the specified information to the system log.
*
* Testing:
*   Print three lines of text.
--*/

void os_print (UINT8  *p_string)  
   {
   char c;
   
   while (c = *p_string++)
      putchar (c);
   }                 

/*++
* ===================
* = os_buff_hdr_get =
* ===================
*
* Description:
*   For simulation, all packets are in flat buffer space, so straight 
*   copies can be done.
--*/

void os_buff_hdr_get (void   *p_pkt,
                      UINT16 *p_lec_hdr,
                      UINT32 *p_bytes_copied)
   {
   memcpy (p_lec_hdr, p_pkt, sizeof (*p_lec_hdr));
   *p_bytes_copied = sizeof (*p_lec_hdr);
   }

/*++
* ====================
* = os_buff_hdr_copy =
* ====================
*
* Description:
*   For simulation, all packets are in flat buffer space, so straight 
*   copies can be done.
--*/

void os_buff_hdr_copy (void   *p_pkt,
                       UINT32  pkt_len,
                       void   *p_dst,
                       UINT32  dst_len,
                       UINT32 *p_bytes_copied)
   {
   UINT32 bytes_to_copy;

   /* Determine the number of bytes to copy and copy them. */

   bytes_to_copy = MIN (pkt_len, dst_len);
   memcpy (p_dst, p_pkt, bytes_to_copy);
   
   /* Return the number of bytes copied. */

   *p_bytes_copied = bytes_to_copy;
   }

